/***************************************************************************
 *   Copyright (C) 2005 by Berislav Kovacki                                *
 *   beca@sezampro.yu                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#ifdef GP32
   #include <x_gp32.h>
#endif

#include <stdlib.h>
#include <SDL.h>
#include <SDL_mixer.h>
#include "interlogic.h"
#include "mainmenu.h"
#include "splash.h"
#include "password.h"
#include "levels.h"
#include "credits.h"

#define STATE_STARTUP   0
#define STATE_SPLASH    1
#define STATE_MAINMENU  2
#define STATE_LEVELS    3
#define STATE_CREDITS   4
#define STATE_PASSWORD  5

#define SDL_PushQuitEvent() {SDL_Event event; \
            event.type = SDL_QUIT; SDL_PushEvent(&event); }

static const char _program_title[] = "InterLOGIC V0.2";

static int interlogic_state = STATE_STARTUP;

SDL_Surface* interlogic_initialize_video();
int interlogic_keyboard(SDL_Surface* screen, Uint8 state, SDLKey key);
int interlogic_process_events(SDL_Surface* screen);
int interlogic_render(SDL_Surface* screen);
int interlogic_exit_state(SDL_Surface* screen);

Mix_Chunk *sonido_ball = NULL;		//Pointer to our sound, in memory
Mix_Chunk *sonido_complete = NULL;
Mix_Chunk *sonido_timesup = NULL;
Mix_Chunk *sonido_menusel = NULL;

Mix_Music *mus_title, *mus_game;

FILE *file_pass;

SDL_Surface* interlogic_initialize_video()
{
  Uint32 video_flags = SDL_HWSURFACE | SDL_HWPALETTE | SDL_DOUBLEBUF;
  Uint32 video_bits = 16;
  #ifdef GP32
	x_gp32_SetCPUSpeed_133();
	video_flags |= SDL_FULLSCREEN;
  #endif


  SDL_Surface*   screen = NULL;
  SDL_VideoInfo* pSDLVideoInfo;

  pSDLVideoInfo = SDL_GetVideoInfo();

  screen = SDL_SetVideoMode(320, 240,
    video_bits,
    video_flags);

  SDL_WM_SetCaption(_program_title, _program_title);

  return screen;
}

int interlogic_process_events(SDL_Surface* screen)
{
  SDL_Event      event;
  int bExitGame = 0;

  while (SDL_PollEvent(&event))
  {
    switch (event.type)
    {
      case SDL_KEYDOWN:
        switch (event.key.keysym.sym)
        {
          case SDLK_F4:
            SDL_WM_ToggleFullScreen(screen);
            break;

          default:
            break;
        }
      case SDL_KEYUP:
        interlogic_keyboard(screen, event.key.state,
          event.key.keysym.sym);
        break;

      case SDL_QUIT:
        bExitGame = 1;
        break;
    }
  }

  return bExitGame;
}

int interlogic_render(SDL_Surface* screen)
{
  switch (interlogic_state)
  {
    case STATE_SPLASH:   splash_render(screen);   break;
    case STATE_MAINMENU: mainmenu_render(screen); break;
    case STATE_LEVELS:   levels_render(screen);   break;
    case STATE_PASSWORD: password_render(screen); break;
    case STATE_CREDITS:  credits_render(screen);  break;
    default: return 0;
  }

  return SDL_Flip(screen);
}

int interlogic_exit_state(SDL_Surface* screen)
{
  switch (interlogic_state)
  {
    case STATE_SPLASH:   splash_uninitialize(screen);   break;
    case STATE_MAINMENU: mainmenu_uninitialize(screen); break;
    case STATE_LEVELS:   levels_uninitialize(screen);   break;
    case STATE_PASSWORD: password_uninitialize(screen); break;
    case STATE_CREDITS:  credits_uninitialize(screen);  break;
    case STATE_STARTUP: break;
    default: return 0;
  }

  return -1;
}

int interlogic_keyboard(SDL_Surface* screen, Uint8 state, SDLKey key)
{
  switch (interlogic_state)
  {
    case STATE_SPLASH:   splash_keyboard(screen, state, key);   break;
    case STATE_MAINMENU: mainmenu_keyboard(screen, state, key); break;
    case STATE_LEVELS:   levels_keyboard(screen, state, key);   break;
    case STATE_PASSWORD: password_keyboard(screen, state, key); break;
    case STATE_CREDITS:  credits_keyboard(screen, state, key);  break;
    default: return 0;
  }

  return -1;
}

int interlogic_switch_state(SDL_Surface* screen, int code)
{
  interlogic_exit_state(screen);
  int ind;

  switch (interlogic_state)
  {
    case STATE_STARTUP:
      interlogic_state = STATE_SPLASH;
      splash_initialize(screen);
      break;

    case STATE_SPLASH:
      interlogic_state = STATE_MAINMENU;
      mainmenu_initialize(screen);
      break;

    case STATE_MAINMENU:
      switch (code)
      {
        case 0:
          interlogic_state = STATE_LEVELS;
          levels_initialize(screen, 0);
          break;

        case 1:
	  interlogic_state = STATE_LEVELS;
          fseek (file_pass, 0, 0);
          ind=fgetc(file_pass);
          interlogic_switch_state(screen, ind);

          /*interlogic_state = STATE_PASSWORD;
          password_initialize(screen);
          */
          break;

        case 2:
          interlogic_state = STATE_CREDITS;
          credits_initialize(screen);
          break;

        case 3:
          interlogic_state = STATE_STARTUP;
          SDL_PushQuitEvent();
          break;
      }
      break;

    case STATE_CREDITS:
      interlogic_state = STATE_MAINMENU;
      mainmenu_initialize(screen);
      break;

    case STATE_PASSWORD:
      if (code == -1)
      {
        interlogic_state = STATE_MAINMENU;
        mainmenu_initialize(screen);
      }
      else
      {
        interlogic_state = STATE_LEVELS;
        levels_initialize(screen, code);
      }
      break;

    case STATE_LEVELS:
      if (code == -1)
      {
        interlogic_state = STATE_MAINMENU;
        mainmenu_initialize(screen);
      }
      else
      {
        interlogic_state = STATE_LEVELS;
        levels_initialize(screen, code);
      }
      break;
  }

  return -1;
}

int main(int argc, char *argv[])
{
  SDL_Surface*   screen;

  if (SDL_Init(SDL_INIT_VIDEO) < 0)
  {
    SDL_Quit();
  }

  if (!(screen = interlogic_initialize_video()))
  {
    SDL_Quit();
  }

	int channel;				//Channel on which our sound is played
	  
	int audio_rate = 22050;			//Frequency of audio playback
	Uint16 audio_format = AUDIO_S16; 	//Format of the audio we're playing
	int audio_channels = 2;			//2 channels = stereo
	int audio_buffers = 2048;		//Size of the audio buffers in memory
	

	//Initialize SDL_mixer with our chosen audio settings
	if(Mix_OpenAudio(audio_rate, audio_format, audio_channels, audio_buffers) != 0) {
		printf("Unable to initialize audio: %s\n", Mix_GetError());
		exit(1);
	}
	
	//Load our WAV file from disk
	sonido_ball = Mix_LoadWAV(SONIDO_BALL);
	sonido_complete = Mix_LoadWAV(SONIDO_COMPLETE);
	sonido_timesup = Mix_LoadWAV(SONIDO_TIMESUP);
	sonido_menusel = Mix_LoadWAV(SONIDO_MENUSEL);	
  	mus_game = Mix_LoadMUS(MUS_GAME); 
  	//mus_title = Mix_LoadMUS(MUS_TITLE);   
	
	//Mix_PlayChannel(-1, sonido_ball, -1); 
	
	if(channel == -1) {
		printf("Unable to play WAV file: %s\n", Mix_GetError());
	}
	
	  file_pass = fopen (FICHERO_PASSWORD, "r+");
      	  if (!file_pass){
      	  	file_pass = fopen (FICHERO_PASSWORD, "w+");
      	  	fputc (0, file_pass);
      	  	}

	
	//Release the memory allocated to our sound
	//Mix_FreeChunk(sound);
	
	//Need to make sure that SDL_mixer and SDL have a chance to clean up
	//Mix_CloseAudio(); 	

  interlogic_switch_state(screen, 0);
  Mix_PlayMusic(mus_game, -1);


  while (!interlogic_process_events(screen))
  {
    interlogic_render(screen);
  }

  interlogic_exit_state(screen);
	
  //Need to make sure that SDL_mixer and SDL have a chance to clean up
  Mix_CloseAudio(); 
  fclose (file_pass);	


  SDL_Quit();
  return 0;
}
